@@ -62,6 +62,14 @@ PyTypeObject PyGTypeWrapper_Type = {
     NULL
 };
 
+/**
+ * pyg_type_wrapper_new:
+ * type: a GType
+ *
+ * Creates a Python wrapper for a GType.
+ *
+ * Returns: the Python wrapper.
+ */
 PyObject *
 pyg_type_wrapper_new(GType type)
 {
@@ -76,6 +84,15 @@ pyg_type_wrapper_new(GType type)
     return (PyObject *)self;
 }
 
+/**
+ * pyg_type_from_object:
+ * obj: a Python object
+ *
+ * converts a python object to a GType.  Raises an exception if it
+ * can't perform the conversion.
+ *
+ * Returns: the corresponding GType, or 0 on error.
+ */
 GType
 pyg_type_from_object(PyObject *obj)
 {
@@ -136,6 +153,19 @@ pyg_type_from_object(PyObject *obj)
 
 /* -------------- GValue marshalling ------------------ */
 
+/**
+ * pyg_enum_get_value:
+ * @enum_type: the GType of the flag.
+ * @obj: a Python object representing the flag value
+ * @val: a pointer to the location to store the integer representation of the flag.
+ *
+ * Converts a Python object to the integer equivalent.  The conversion
+ * will depend on the type of the Python object.  If the object is an
+ * integer, it is passed through directly.  If it is a string, it will
+ * be treated as a full or short enum name as defined in the GType.
+ *
+ * Returns: 0 on success or -1 on failure
+ */
 gint
 pyg_enum_get_value(GType enum_type, PyObject *obj, gint *val)
 {
@@ -178,6 +208,21 @@ pyg_enum_get_value(GType enum_type, PyObject *obj, gint *val)
     return res;
 }
 
+/**
+ * pyg_flags_get_value:
+ * @flag_type: the GType of the flag.
+ * @obj: a Python object representing the flag value
+ * @val: a pointer to the location to store the integer representation of the flag.
+ *
+ * Converts a Python object to the integer equivalent.  The conversion
+ * will depend on the type of the Python object.  If the object is an
+ * integer, it is passed through directly.  If it is a string, it will
+ * be treated as a full or short flag name as defined in the GType.
+ * If it is a tuple, then the items are treated as strings and ORed
+ * together.
+ *
+ * Returns: 0 on success or -1 on failure
+ */
 gint
 pyg_flags_get_value(GType flag_type, PyObject *obj, gint *val)
 {
@@ -260,6 +305,19 @@ static GQuark pyg_boxed_marshal_key = 0;
 #define pyg_boxed_lookup(boxed_type) \
   ((PyGBoxedMarshal *)g_type_get_qdata((boxed_type), pyg_boxed_marshal_key))
 
+
+/**
+ * pyg_register_boxed_custom:
+ * @boxed_type: the GType for boxed type
+ * @from_func: a function to convert GValues to Python objects
+ * @to_func: a function to convert Python objects to GValues
+ *
+ * The standard way of wrapping boxed types in PyGTK is to create a
+ * subclass of gobject.GBoxed and register it with
+ * pyg_register_boxed().  In some cases however, it is useful to have
+ * fine grained control over how a particular type is represented in
+ * Python.  This function allows you to register such a handler.
+ */
 void
 pyg_register_boxed_custom(GType boxed_type,
 			  fromvaluefunc from_func,
@@ -276,6 +334,18 @@ pyg_register_boxed_custom(GType boxed_type,
     g_type_set_qdata(boxed_type, pyg_boxed_marshal_key, bm);
 }
 
+/**
+ * pyg_value_from_pyobject:
+ * @value: the GValue object to store the converted value in.
+ * @obj: the Python object to convert.
+ *
+ * This function converts a Python object and stores the result in a
+ * GValue.  The GValue must be initialised in advance with
+ * g_value_init().  If the Python object can't be converted to the
+ * type of the GValue, then an error is returned.
+ *
+ * Returns: 0 on success, -1 on error.
+ */
 int
 pyg_value_from_pyobject(GValue *value, PyObject *obj)
 {
@@ -425,37 +495,43 @@ pyg_value_from_pyobject(GValue *value, PyObject *obj)
     case G_TYPE_STRING:
 	if (obj == Py_None)
 	    g_value_set_string(value, NULL);
-	else if ((tmp = PyObject_Str(obj)))
+	else if ((tmp = PyObject_Str(obj))) {
 	    g_value_set_string(value, PyString_AsString(tmp));
-	else {
+	    Py_DECREF(tmp);
+	} else {
 	    PyErr_Clear();
 	    return -1;
 	}
-	Py_DECREF(tmp);
 	break;
     case G_TYPE_POINTER:
-	if (PyCObject_Check(obj))
+	if (obj == Py_None)
+	    g_value_set_pointer(value, NULL);
+	else if (PyObject_TypeCheck(obj, &PyGPointer_Type) &&
+		   G_VALUE_HOLDS(value, ((PyGPointer *)obj)->gtype))
+	    g_value_set_pointer(value, pyg_pointer_get(obj, gpointer));
+	else if (PyCObject_Check(obj))
 	    g_value_set_pointer(value, PyCObject_AsVoidPtr(obj));
 	else
 	    return -1;
 	break;
-    case G_TYPE_BOXED:
-	{
-	    PyGBoxedMarshal *bm;
+    case G_TYPE_BOXED: {
+	PyGBoxedMarshal *bm;
 
-	    if (G_VALUE_HOLDS(value, PY_TYPE_OBJECT)) {
-		g_value_set_boxed(value, obj);
-	    } else if (PyObject_TypeCheck(obj, &PyGBoxed_Type) &&
-		       G_VALUE_HOLDS(value, ((PyGBoxed *)obj)->gtype)) {
-		g_value_set_boxed(value, pyg_boxed_get(obj, gpointer));
-	    } else if ((bm = pyg_boxed_lookup(G_VALUE_TYPE(value))) != NULL) {
-		return bm->tovalue(value, obj);
-	    } else if (PyCObject_Check(obj)) {
-		g_value_set_boxed(value, PyCObject_AsVoidPtr(obj));
-	    } else
-		return -1;
-	}
+	if (obj == Py_None)
+	    g_value_set_boxed(value, NULL);
+	if (G_VALUE_HOLDS(value, PY_TYPE_OBJECT))
+	    g_value_set_boxed(value, obj);
+	else if (PyObject_TypeCheck(obj, &PyGBoxed_Type) &&
+		   G_VALUE_HOLDS(value, ((PyGBoxed *)obj)->gtype))
+	    g_value_set_boxed(value, pyg_boxed_get(obj, gpointer));
+	else if ((bm = pyg_boxed_lookup(G_VALUE_TYPE(value))) != NULL)
+	    return bm->tovalue(value, obj);
+	else if (PyCObject_Check(obj))
+	    g_value_set_boxed(value, PyCObject_AsVoidPtr(obj));
+	else
+	    return -1;
 	break;
+    }
     case G_TYPE_PARAM:
 	if (PyGParamSpec_Check(obj))
 	    g_value_set_param(value, PyCObject_AsVoidPtr(obj));
@@ -463,14 +539,14 @@ pyg_value_from_pyobject(GValue *value, PyObject *obj)
 	    return -1;
 	break;
     case G_TYPE_OBJECT:
-	if (!PyObject_TypeCheck(obj, &PyGObject_Type)) {
+	if (obj == Py_None) {
+	    g_value_set_object(value, NULL);
+	} else if (PyObject_TypeCheck(obj, &PyGObject_Type) &&
+		   G_TYPE_CHECK_INSTANCE_TYPE(pygobject_get(obj),
+					      G_VALUE_TYPE(value))) {
+	    g_value_set_object(value, pygobject_get(obj));
+	} else
 	    return -1;
-	}
-	if (!G_TYPE_CHECK_INSTANCE_TYPE(pygobject_get(obj),
-					G_VALUE_TYPE(value))) {
-	    return -1;
-	}
-	g_value_set_object(value, pygobject_get(obj));
 	break;
     default:
 	break;
@@ -499,18 +575,19 @@ pyg_value_as_pyobject(const GValue *value, gboolean copy_boxed)
 	    return pygobject_new(g_value_get_object(value));
 	else
 	    break;
-    case G_TYPE_CHAR:
-	{
-	    gint8 val = g_value_get_char(value);
-	    return PyString_FromStringAndSize((char *)&val, 1);
-	}
-    case G_TYPE_UCHAR:
-	{
-	    guint8 val = g_value_get_uchar(value);
-	    return PyString_FromStringAndSize((char *)&val, 1);
-	}
-    case G_TYPE_BOOLEAN:
-	return PyInt_FromLong(g_value_get_boolean(value));
+    case G_TYPE_CHAR: {
+	gint8 val = g_value_get_char(value);
+	return PyString_FromStringAndSize((char *)&val, 1);
+    }
+    case G_TYPE_UCHAR: {
+	guint8 val = g_value_get_uchar(value);
+	return PyString_FromStringAndSize((char *)&val, 1);
+    }
+    case G_TYPE_BOOLEAN: {
+	PyObject *val = g_value_get_boolean(value) ? Py_True : Py_False;
+	Py_INCREF(val);
+	return val;
+    }
     case G_TYPE_INT:
 	return PyInt_FromLong(g_value_get_int(value));
     case G_TYPE_UINT:
@@ -572,32 +649,32 @@ pyg_value_as_pyobject(const GValue *value, gboolean copy_boxed)
 	    return Py_None;
 	}
     case G_TYPE_POINTER:
-	return PyCObject_FromVoidPtr(g_value_get_pointer(value), NULL);
-    case G_TYPE_BOXED:
-	{
-	    PyGBoxedMarshal *bm;
+	return pyg_pointer_new(G_VALUE_TYPE(value),
+			       g_value_get_pointer(value));
+    case G_TYPE_BOXED: {
+	PyGBoxedMarshal *bm;
 
-	    if (G_VALUE_HOLDS(value, PY_TYPE_OBJECT)) {
-		PyObject *ret = (PyObject *)g_value_dup_boxed(value);
-		if (ret == NULL) {
-		    Py_INCREF(Py_None);
-		    return Py_None;
-		}
-		return ret;
-	    }
-	    
-	    bm = pyg_boxed_lookup(G_VALUE_TYPE(value));
-	    if (bm) {
-		return bm->fromvalue(value);
-	    } else {
-		if (copy_boxed)
-		    return pyg_boxed_new(G_VALUE_TYPE(value),
-					 g_value_get_boxed(value), TRUE, TRUE);
-		else
-		    return pyg_boxed_new(G_VALUE_TYPE(value),
-					 g_value_get_boxed(value),FALSE,FALSE);
+	if (G_VALUE_HOLDS(value, PY_TYPE_OBJECT)) {
+	    PyObject *ret = (PyObject *)g_value_dup_boxed(value);
+	    if (ret == NULL) {
+		Py_INCREF(Py_None);
+		return Py_None;
 	    }
+	    return ret;
 	}
+	    
+	bm = pyg_boxed_lookup(G_VALUE_TYPE(value));
+	if (bm) {
+	    return bm->fromvalue(value);
+	} else {
+	    if (copy_boxed)
+		return pyg_boxed_new(G_VALUE_TYPE(value),
+				     g_value_get_boxed(value), TRUE, TRUE);
+	    else
+		return pyg_boxed_new(G_VALUE_TYPE(value),
+				     g_value_get_boxed(value),FALSE,FALSE);
+	}
+    }
     case G_TYPE_PARAM:
 	return pyg_param_spec_new(g_value_get_param(value));
     case G_TYPE_OBJECT:
@@ -683,6 +760,18 @@ pyg_closure_marshal(GClosure *closure,
     pyg_unblock_threads();
 }
 
+/**
+ * pyg_closure_new:
+ * callback: a Python callable object
+ * extra_args: a tuple of extra arguments, or None/NULL.
+ * swap_data: an alternative python object to pass first.
+ *
+ * Creates a GClosure wrapping a Python callable and optionally a set
+ * of additional function arguments.  This is needed to attach python
+ * handlers to signals, for instance.
+ *
+ * Returns: the new closure.
+ */
 GClosure *
 pyg_closure_new(PyObject *callback, PyObject *extra_args, PyObject *swap_data)
 {
@@ -800,6 +889,15 @@ pyg_signal_class_closure_marshal(GClosure *closure,
     pyg_unblock_threads();
 }
 
+/**
+ * pyg_signal_class_closure_get:
+ *
+ * Returns the GClosure used for the class closure of signals.  When
+ * called, it will invoke the method do_signalname (for the signal
+ * "signalname").
+ *
+ * Returns: the closure.
+ */
 GClosure *
 pyg_signal_class_closure_get(void)
 {
@@ -995,6 +1093,15 @@ static PyTypeObject PyGObjectDoc_Type = {
     (descrsetfunc)0
 };
 
+/**
+ * pyg_object_descr_doc_get:
+ *
+ * Returns an object intended to be the __doc__ attribute of GObject
+ * wrappers.  When read in the context of the object it will return
+ * some documentation about the signals and properties of the object.
+ *
+ * Returns: the descriptor.
+ */
 PyObject *
 pyg_object_descr_doc_get(void)
 {
